(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2004-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: io.ml,v 1.2 2011-02-23 17:43:29 fclement Exp $ *)

open Communication;;
open Typed_value;;

(* {3 General send communication functions on a buffer} *)

let send_communication = Write_communication.write_communication;;

let send_values ob vals = send_communication ob (Phrase vals);;

let send_one_value ob v = send_values ob [| v |];;

(* {6 Specialized send communication functions on a buffer} *)

let gen_send of_native ob t =
  Write_typed_value.write_typed_value ob (of_native t);;

let send_string = gen_send of_string
and send_int = gen_send of_int
and send_int_couple = gen_send of_int_couple
and send_int_triple = gen_send of_int_triple
and send_float = gen_send of_float
and send_float_couple = gen_send of_float_couple
and send_float_triple = gen_send of_float_triple
and send_string_vector = gen_send of_string_vector
and send_int_vector = gen_send of_int_vector
and send_int_couple_vector = gen_send of_int_couple_vector
and send_int_triple_vector = gen_send of_int_triple_vector
and send_float_vector = gen_send of_float_vector
and send_float_couple_vector = gen_send of_float_couple_vector
and send_float_triple_vector = gen_send of_float_triple_vector
and send_c_float_matrix = gen_send of_c_float_matrix
and send_fortran_float_matrix = gen_send of_fortran_float_matrix
;;

(* General receive communication from a scanning buffer *)

exception Wrong_communication of string * string;;

let receive_communication = Read_communication.read_communication;;

let receive_values ib =
  match receive_communication ib with
  | Phrase vals -> vals
  | Task _ | Result _ | Error _ | Service _ ->
    failwith "Io.receive_values: communication is not a phrase"
;;

let receive_one_value ib =
  let vals = receive_values ib in
  if Array.length vals = 1 then vals.(0) else
  failwith
    "Io.receive_one_value: \
     communication phrase has more than one value"
;;

(* Specialized receive communications from a buffer *)

let gen_receive to_native ib =
  try to_native (Read_typed_value.read_typed_value ib) with
  | Wrong_communication (s1, s2) ->
    prerr_endline
      (Printf.sprintf
        "Io: wrong communication.\
       \nA %s communication was expected but a %s communication was received"
        s1 s2);
    exit 2
  | End_of_file ->
    prerr_endline "Io: premature end of communication";
    exit 2
  | x ->
    prerr_endline
      (Printf.sprintf "Io: spurious exception: %s" (Printexc.to_string x));
    exit 2
;;

let receive_string = gen_receive to_string
and receive_int = gen_receive to_int
and receive_int_couple = gen_receive to_int_couple
and receive_int_triple = gen_receive to_int_triple
and receive_float = gen_receive to_float
and receive_float_couple = gen_receive to_float_couple
and receive_float_triple = gen_receive to_float_triple
and receive_string_vector = gen_receive to_string_vector
and receive_int_vector = gen_receive to_int_vector
and receive_int_couple_vector = gen_receive to_int_couple_vector
and receive_int_triple_vector = gen_receive to_int_triple_vector
and receive_float_vector = gen_receive to_float_vector
and receive_float_couple_vector = gen_receive to_float_couple_vector
and receive_float_triple_vector = gen_receive to_float_triple_vector
and receive_c_float_matrix = gen_receive to_c_float_matrix
and receive_fortran_float_matrix = gen_receive to_fortran_float_matrix
;;
