(***********************************************************************)
(*                                                                     *)
(*                                Sklml                                *)
(*                                                                     *)
(*                 The OCaml parallel skeleton system                  *)
(*                                                                     *)
(*                  projet Estime, INRIA-Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 2007-2011 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Quentin Carbonneaux <Quentin.Carbonneaux@inria.fr>                 *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: sk.mli,v 1.25.2.1 2011-09-21 13:08:03 fclement Exp $ *)

(** {3 Computation trees manipulations } *)

type ('a, 'b) sum = ('a, 'b) Sklstream.sum = Inl of 'a | Inr of 'b;;

module Eq : sig
  type ('a, 'b) t
  val refl : unit -> ('a, 'a) t
  val app : ('a, 'b) t -> 'a -> 'b
end
;;

module rec Skl : sig
  module type ATOM = sig
    type a and b
    type x

    val color : int
    val params : x
    val func : (x, a, b) Expor.llexpr
  end
  module type PIPE = sig
    type a and b
    type c

    val sk1 : (a, c) Skl.t
    val sk2 : (c, b) Skl.t
  end
  module type PROD = sig
    type a and b
    type a0 and a1 and b0 and b1

    val eq_in : (a, (a0 * a1)) Eq.t
    val eq_out : ((b0 * b1), b) Eq.t
    val sk1 : (a0, b0) Skl.t
    val sk2 : (a1, b1) Skl.t
  end
  module type SUM = sig
    type a and b
    type aa and ab

    val eq_in : (a, (aa, ab) sum) Eq.t
    val sk1 : (aa, b) Skl.t
    val sk2 : (ab, b) Skl.t
  end

  type ('a, 'b) atom = (module Skl.ATOM with type a = 'a and type b = 'b)
  type ('a, 'b) id = ('a, 'b) Eq.t
  type ('a, 'b) prod = (module Skl.PROD with type a = 'a and type b = 'b)
  type ('a, 'b) pipe = (module Skl.PIPE with type a = 'a and type b = 'b)
  type ('a, 'b) rails = (('a, 'b) Skl.t) array
  type ('a, 'b) sum = (module Skl.SUM with type a = 'a and type b = 'b)
  type ('a, 'b) par = (('a, 'b) Skl.t * ('a, 'b) Skl.t)
  type ('a, 'b) loop = (('a, 'b) Eq.t * ('a, bool) Skl.t * ('a, 'a) Skl.t)

  type ('a, 'b) t =
    | Atom of ('a, 'b) atom
    | Id of ('a, 'b) id
    | Prod of ('a, 'b) prod
    | Pipe of ('a, 'b) pipe
    | Rails of ('a, 'b) rails
    | Sum of ('a, 'b) sum
    | Par  of ('a, 'b) par
    | Loop of ('a, 'b) loop
end
;;

type ('a, 'b, 'd) atom_runner =
  { f_atom : 'x. 'x -> ('x, 'a, 'b) Expor.llexpr -> int -> 'd; }
;;
type ('a, 'b, 'd) id_runner =
  { f_id : ('a -> 'b) -> 'd; }
;;
type ('a, 'b, 'd) prod_runner =
  { f_prod : 'a0 'b0 'a1 'b1.
      ('a -> ('a0 * 'a1)) -> (('b0 * 'b1) -> 'b) ->
      ('a0, 'b0) Skl.t -> ('a1, 'b1) Skl.t -> 'd; }
;;
type ('a, 'c, 'd) pipe_runner =
  { f_pipe : 'b. ('a, 'b) Skl.t -> ('b, 'c) Skl.t -> 'd; }
;;
type ('a, 'b, 'd) par_runner =
  { f_par : ('a, 'b) Skl.t -> ('a, 'b) Skl.t -> 'd; }
;;
type ('a, 'b, 'd) rails_runner =
  { f_rails : (('a, 'b) Skl.t) array -> 'd; }
;;
type ('a, 'b, 'd) sum_runner =
  { f_sum : 'aa 'ab.
      ('a -> ('aa, 'ab) sum) -> ('aa, 'b) Skl.t -> ('ab, 'b) Skl.t -> 'd; }
;;
type ('a, 'b, 'd) loop_runner =
  { f_loop : ('a -> 'b) -> ('a, bool) Skl.t -> ('a, 'a) Skl.t -> 'd; }
;;

(** {6 [Skl.t] exploders } *)

val with_atom : ('a, 'b) Skl.atom -> ('a, 'b, 'd) atom_runner -> 'd;;
val with_id : ('a, 'b) Skl.id -> ('a, 'b, 'd) id_runner -> 'd;;
val with_prod : ('a, 'b) Skl.prod -> ('a, 'b, 'd) prod_runner -> 'd;;
val with_pipe : ('a, 'c) Skl.pipe -> ('a, 'c, 'd) pipe_runner -> 'd;;
val with_par :  ('a, 'b) Skl.par -> ('a, 'b, 'd) par_runner -> 'd;;
val with_rails : ('a, 'b) Skl.rails -> ('a, 'b, 'd) rails_runner -> 'd;;
val with_sum : ('a, 'b) Skl.sum -> ('a, 'b, 'd) sum_runner -> 'd;;
val with_loop : ('a, 'b) Skl.loop -> ('a, 'b, 'd) loop_runner -> 'd;;

(** {6 [Skl.t] constructors } *)

val c_atom : 'x -> ('x, 'a, 'b) Expor.llexpr -> int -> ('a, 'b) Skl.t;;
val c_id : unit -> ('a, 'a) Skl.t
val c_prod :
  ('a0, 'b0) Skl.t -> ('a1, 'b1) Skl.t -> ('a0 * 'a1, 'b0 * 'b1) Skl.t;;
val c_pipe : ('a, 'b) Skl.t -> ('b, 'c) Skl.t -> ('a, 'c) Skl.t;;
val c_par : ('a, 'b) Skl.t -> ('a, 'b) Skl.t -> ('a, 'b) Skl.t;;
val c_rails : (('a, 'b) Skl.t) array -> ('a, 'b) Skl.t;;
val c_sum :
  ('aa, 'b) Skl.t -> ('ab, 'b) Skl.t -> (('aa, 'ab) sum, 'b) Skl.t;;
val c_loop : ('a, bool) Skl.t -> ('a, 'a) Skl.t -> ('a, 'a) Skl.t;;

(** {6 [Skl.Atom] color modifier } *)

val colorize : ('a, 'b) Skl.atom -> int -> ('a, 'b) Skl.t;;
